package org.apache.maven.plugin.compiler.stubs;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.codehaus.plexus.compiler.manager.CompilerManager;
import org.codehaus.plexus.compiler.manager.NoSuchCompilerException;

/**
 * @author Edwin Punzalan
 */
public class CompilerManagerStub
    implements CompilerManager
{
    private boolean shouldFail;

    public CompilerManagerStub()
    {
        this( false );
    }

    public CompilerManagerStub( boolean shouldFail )
    {
        this.shouldFail = shouldFail;
    }

    public org.codehaus.plexus.compiler.Compiler getCompiler( String compilerId )
        throws NoSuchCompilerException
    {
        return new CompilerStub( shouldFail );
    }
}
